use crate::components::*;
use specs::{Builder, World, WorldExt, EntityBuilder};

/// Cut down on the common syntax boilerplate
fn common_entity<'w>(world: &'w mut World, position: Position, z: u8,  sprite_path: &str) -> EntityBuilder<'w> {
    return world
        .create_entity()
        .with(position.with_z(z))
        .with(Renderable { path: sprite_path.to_string() });
}

pub fn create_wall(world: &mut World, position: Position) {
    common_entity(world, position, 10, "/images/wall.png")
        .with(Wall {})
        .with(Immovable)
        .build();
}

pub fn create_floor(world: &mut World, position: Position) {
    common_entity(world, position, 5, "/images/floor.png").build();
}

pub fn create_box(world: &mut World, position: Position) {
    common_entity(world, position, 10, "/images/box.png")
        .with(Box {})
        .with(Movable)
        .build();
}

pub fn create_box_spot(world: &mut World, position: Position) {
    common_entity(world, position, 9, "/images/box_spot.png")
        .with(BoxSpot {})
        .build();
}

pub fn create_player(world: &mut World, position: Position) {
    common_entity(world, position, 10, "/images/player.png")
        .with(Player {})
        .with(Movable)
        .build();
}
